#include <math.h>
#include <stdlib.h>
#include "invsubs.h"

#define PI              3.1415926
 
#define rcmul(x,y)      (x.re*y.re + x.im*y.im)
#define icmul(x,y)      (x.im*y.re - x.re*y.im)
#define  rmul(x,y)      (x.re*y.re - x.im*y.im)
#define  imul(x,y)      (x.im*y.re + x.re*y.im)
 
#define ABSOLUTE(x) ( (x > 0) ? x : -x)
#define MAX(a,b) ( (a) > (b) ? (a) : (b) )
 
#define MINBLOB 10
#define DCMAX 0.25  /* max value of dc level w/r amp */
 


void cfft2d(struct complex *x,int nx)
{
	struct complex *dummy;
	int i, j;
	dummy  = (struct complex *) malloc(sizeof(struct complex)*nx);
	for(i=0;i<nx;i++)
		cfft(&x[nx*i],nx,1);
	for(i=0;i<nx;i++){
		for(j=0;j<nx;j++){
			dummy[j].re = x[j*nx+i].re;
			dummy[j].im = x[j*nx+i].im;
		}
		cfft(dummy,nx,1);
		for(j=0;j<nx;j++){
			x[j*nx+i].re = dummy[j].re/(float)nx;
			x[j*nx+i].im = dummy[j].im/(float)nx;
		}
	}
	free(dummy);
}
void cfft(struct complex *x,int n,int isign)
{
	register struct complex *px, *qx, *rx;
	struct complex *limit, *qlimit, dtemp;
	float cn, sn, cd, sd, temp, real, imag;
	int m, j, istep;
	float *psintab;
	static float sintab[] =
	{
		1.0000000e+00,	/* sin(pi/2) */
		7.0710678e-01,	/* sin(pi/4) */
		3.8268343e-01,	/* sin(pi/8) */
		1.9509032e-01,	/* sin(pi/16) */
		9.8017140e-02,	/* sin(pi/32) */
		4.9067674e-02,	/* sin(pi/64) */
		2.4541228e-02,	/* sin(pi/128) */
		1.2271538e-02,	/* sin(pi/256) */
		6.1358846e-03,	/* sin(pi/512) */
		3.0679568e-03,	/* sin(pi/1024) */
		1.5339802e-03,	/* sin(pi/2048) */
		7.6699032e-04,	/* sin(pi/4096) */
		3.8349519e-04,	/* sin(pi/8192) */
		1.9174760e-04	/* sin(pi/16384) */
	};

	limit= x + n;
	j= 0;
	for(px=x; px<limit; px++)
	{
		if(px < (qx= x+j))
		{
			dtemp= *qx;
			*qx= *px;
			*px= dtemp;
		}
		m = n>>1;
		while( m>=1 && j>=m )
		{
			j-= m;
			m>>= 1;
		}
		j+= m;
	}
	rx= x+1;
	for(px=x; px<limit; px+= 2, rx+= 2)
	{
		temp= rx->re;
		rx->re= px->re -temp;
		px->re += temp;
		temp= rx->im;
		rx->im= px->im -temp;
		px->im += temp;
	}
	j=2;
	psintab= sintab;
	while( j < n )
	{
		istep= j<<1;
		sd= *psintab++;
		temp= *psintab;
		cd= 2.0 * temp * temp;
		cn= 1.0;
		sn= 0.0;
		if( isign > 0 ) sd= -sd;
		/* sign convention changed to conform with
				 * array processor
				 */
		qlimit= x+j;
		for(qx=x; qx< qlimit; qx++)
		{
			for(px=qx; px<limit; px+= istep)
			{
				rx= px + j;
				real= cn * rx->re - sn * rx->im;
				imag= sn * rx->re + cn * rx->im;
				rx->re = px->re - real;
				rx->im = px->im - imag;
				px->re += real;
				px->im += imag;
			}
			temp= cd * cn + sd * sn;
			sn += (sd * cn - cd * sn);
			cn -= temp;
		}
		j= istep;
	}
	return;
}
