/*
 * function to calculate the great circle distance and back azimuth (azimuth
 * clockwise from north between two points on a sphere with a flattening of
 * 1/295)
 *
 * returns the azimuth from point a to point b, measured from north
 * 
 * all IO is in degrees
 * 
 */

#include <math.h>
#include "invsubs.h"

double          flat = 1.0000000;
double          rad;

#define cl 0			/* subscripts for sine and cosine arrays */
#define sl 1
#define cn 2
#define sn 3

double          cdif;


void delta_az(float *alati, float *aloni, float *blati, float *bloni, float *delta, float *baz)
{
	double          a[4], b[4];
	double          alat, alon, blat, blon;

	rad = 3.14159265358979323846 / 180;

	alat = (double) *alati;
	alon = (double) *aloni;
	blat = (double) *blati;
	blon = (double) *bloni;

	dbpt(alat, alon, a);
	dbpt(blat, blon, b);

	cdif = b[cn] * a[cn] + b[sn] * a[sn];
	*delta = acos(a[sl] * b[sl] * cdif + a[cl] * b[cl]) / rad;

	*baz = atan2((a[cn] * b[sn] - b[cn] * a[sn]) * b[sl], a[sl] * b[cl] - a[cl] * cdif * b[sl]);
	*baz /= rad;
	if (*baz < 0)
		*baz += 360;

}

double          maxDegree = 360;

void dbpt(double lat, double lon, double a[4])
{
	a[cl] = 1;
	if (fabs(lat != 90))
		a[cl] = sin(atan(flat * tan(lat * rad)));
	a[sl] = sqrt(1 - a[cl] * a[cl]);
	a[sn] = sin(lon * rad);

	cdif = fmod(lon, maxDegree);
	if (cdif < 0)
		cdif *= -1;

	a[cn] = sqrt(1 - a[sn] * a[sn]);
	if (cdif > 90 && cdif < 270)
		a[cn] *= -1;

}
