#include <stdio.h>
#include "geoggeoccnvc.h"
#include <math.h>

/* 13 NOV 2017 
 * read the output of the GETALL script in ALLt
 * and convert from geographic to geocentric
 *
 * 14 NOV 2017 
 * adjust the dispersion velocity to the new coordinate system since the distance changes
 * We compute the arc distance and distance = 111.19 * acr distance to be compatible with
 * nmulti_lsqr.  We preserve the travel time which is the observed parameter. 
*/
void delta_az(float *alati, float *aloni, float *blati, float *bloni, float *delta, float *baz);
void dbpt(double lat, double lon, double *a);

void main(void)
{
char istr[10], lorr[2];
float lat1,lon1,lat2,lon2,vel,period,dvel,dist;
double gclat1,gclon1,gclat2,gclon2;
float delta, baz;
float newvel;
float newdist;
while ( scanf("%s %f %f %f %f %f %f %f  %s %f",istr,&lat1,&lon1,&lat2,&lon2,&vel,&period,&dvel,lorr, &dist) != EOF){

delta_az( &lat1, &lon1, &lat2, &lon2, &delta, &baz);
geog_to_geoc((double)lat1,(double)lon1,&gclat1,&gclon1) ;
geog_to_geoc((double)lat2,(double)lon2,&gclat2,&gclon2) ;
newdist = delta* 111.19;
newvel = vel*newdist/dist;
printf("%s %9.4f %9.4f %9.4f %9.4f %7.4f %f %f %s %f\n",istr,gclat1,gclon1,gclat2,gclon2,newvel,period,dvel,lorr,newdist);
}
}

double          flat = 1.0000000;
double          rad;


#define cl 0                    /* subscripts for sine and cosine arrays */
#define sl 1
#define cn 2
#define sn 3

double          cdif;


void delta_az(float *alati, float *aloni, float *blati, float *bloni, float *delta, float *baz)
{
        double          a[4], b[4];
        double          alat, alon, blat, blon;

        rad = 3.14159265358979323846 / 180;

        alat = (double) *alati;
        alon = (double) *aloni;
        blat = (double) *blati;
        blon = (double) *bloni;

        dbpt(alat, alon, a);
        dbpt(blat, blon, b);

        cdif = b[cn] * a[cn] + b[sn] * a[sn];
        *delta = acos(a[sl] * b[sl] * cdif + a[cl] * b[cl]) / rad;

        *baz = atan2((a[cn] * b[sn] - b[cn] * a[sn]) * b[sl], a[sl] * b[cl] - a[cl] * cdif * b[sl]);
        *baz /= rad;
        if (*baz < 0)
                *baz += 360;

}

double          maxDegree = 360;

void dbpt(double lat, double lon, double a[4])
{
        a[cl] = 1;
        if (fabs(lat != 90))
                a[cl] = sin(atan(flat * tan(lat * rad)));
        a[sl] = sqrt(1 - a[cl] * a[cl]);
        a[sn] = sin(lon * rad);

        cdif = fmod(lon, maxDegree);
        if (cdif < 0)
                cdif *= -1;

        a[cn] = sqrt(1 - a[sn] * a[sn]);
        if (cdif > 90 && cdif < 270)
                a[cn] *= -1;

}

